/*
 * Copyright (C) 2017-2021 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LOGGER_HPP_
#define LOGGER_HPP_

#include "compiler.hpp"

#if __has_include("print.hpp")
#include "print.hpp"
#else
#include "output.hpp"
#endif

#include <syslog.h>
#include <sstream>

namespace server {
struct logger final {
    class info : public std::ostringstream {
    public:
        auto operator=(const info&) -> info& = delete;
        info(const info&) = delete;

        info() = default;
        ~info() override;
    };

    class notice : public std::ostringstream {
    public:
        auto operator=(const notice&) -> notice& = delete;
        notice(const notice&) = delete;

        notice() = default;
        ~notice() override;
    };

    class debug : public std::ostringstream {
    public:
        auto operator=(const debug&) -> debug& = delete;
        debug(const debug&) = delete;

        debug(unsigned level = 0) : current_(level) {}
        ~debug() override;

        inline static void level(unsigned value) {
            level_ = value;
        }

        inline static auto level() {
            return level_;
        }

    private:
        unsigned current_;
        static unsigned level_;
    };

    class error : public std::ostringstream {
    public:
        auto operator=(const error&) -> error& = delete;
        error(const error&) = delete;

        error() = default;
        ~error() override;
    };

    class warn : public std::ostringstream {
    public:
        auto operator=(const warn&) -> warn& = delete;
        warn(const warn&) = delete;

        warn() = default;
        ~warn() override;
    };

    class crit final : public std::ostringstream {
    public:
        auto operator=(const crit&) -> crit& = delete;
        crit(const crit&) = delete;

        explicit crit(int code) noexcept : excode(code) {}
        [[noreturn]] ~crit() final;

    private:
        int excode;
    };

    static void init(const char *ident, int level, bool console);
};
} // end namespace server

/*!
 * Server logging and output.  This makes it easy to write
 * output and fifo control messages using C++ stream operators.
 * \file logger.hpp
 */
#endif
