/*
 * Copyright (C) 2021 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UTILS_HPP_
#define UTILS_HPP_

#include "compiler.hpp"
#include "print.hpp"

#include <list>
#include <climits>
#include <syslog.h>

namespace server {
class dir final
{
public:
    dir(const dir&) = delete;
    auto operator=(const dir&) -> dir& = delete;

    explicit dir(const std::string& path) noexcept;
    dir(dir&& from) noexcept;
    dir() noexcept;
    ~dir();

    inline explicit operator bool() const {
        return state != nullptr;
    }

    inline auto operator!() const {
        return state == nullptr;
    }

    inline auto operator*() -> std::string {
        return get();
    }

    auto operator=(dir&& from) noexcept -> dir&;
    void scan(const std::string& path);
    auto get() -> std::string;

private:
    struct stateinfo;   // mechanics hidden...
    struct stateinfo *state;

    void release();
    void next();
};
} // end namespace server

namespace server::util {
auto get_env(const char *id, size_t max = FILENAME_MAX) -> const char *;
auto u8verify(const char *cp, size_t max = 255) -> size_t;
auto dirname(const std::string& path) -> std::string;
auto basename(const std::string& path, std::initializer_list<std::string_view> extensions = {}) -> std::string;
auto exists(const std::string& path) -> bool;
auto is_dir(const std::string& path) -> bool;
auto to_b64(const uint8_t *data, size_t size) -> std::string;
auto random_fill(uint8_t *data, size_t size) -> bool;
} // end namespace server utils

/*!
 * Utility functions and support for the server.
 * \file utils.hpp
 */
#endif
